/***************************************************************************
 *   Copyright (C) 2003 by Peter Hess                                      *
 *   peter.hess@ph-home.de                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "qmeterbar.h"
#include <qpainter.h>
#include <qstring.h>
#include <qlayout.h>
#include <qfontmetrics.h>
#include <qtooltip.h>

//#include <kmessagebox.h>

QMeterBar::QMeterBar(int min, int max, qm_mode mode, QWidget *parent, const char *name, WFlags f)
 : QWidget(parent, name, f)
{
	signal = min;
	minVal = min;
	maxVal = max;
	minSignal = 255;
	maxSignal = -255;
	visibleMinMax = false;
	setColors(QBrush::red, QBrush::yellow, QBrush::green);
	fontColor=QBrush::black;
	setColorLevels(33,66);
	indicatorFont = QFont("Helvetica",6);
	QMeterBar::mode = mode;
	shortText = false;
	switch (mode) {
		case qm_pro: postfix = "%"; break;
		case qm_dbm: postfix = " dBm"; break;
		default: postfix = "";
	}
	font.setFamily("Helvetica");
}

QMeterBar::~QMeterBar()
{
}

/*!
    \fn QMeterBar::paintEvent()
 */
void QMeterBar::paintEvent(QPaintEvent *)
{
	calcFontSize();
	int h = height();
	int w = width()-2;
	int minSig = (int)((minSignal-minVal)*stepSize);
	int maxSig = (int)((maxSignal-minVal)*stepSize);
	sigLen = (signal-minVal)*stepSize;
	if ( sigLen > w ) sigLen = w;
	QPainter p;
	QBrush b(QBrush::white);
	p.begin(this);
	p.setBrush(b);
#if QT_VERSION >= 0x030201
	if (shortText) font.setStretch(QFont::ExtraCondensed);
	else font.setStretch(QFont::Unstretched);
#endif
	p.setPen(fontColor);
	if ( sigLen < w*colorLevels.middleLevel/100 )
		b.setColor(colorLevels.lowColor);
	else if ( sigLen < w*colorLevels.highLevel/100 )
		b.setColor(colorLevels.middleColor);
	else b.setColor(colorLevels.highColor);
	p.drawRect(rect());
	if (sigLen>=0)p.fillRect(1,1,(int)sigLen,h-2,b);
	if ( visibleMinMax ) {
		p.setFont(indicatorFont);
		QString min = QString::number(minSignal);
		if (minSig < maxSig) {
			p.drawPoint(minSig,h-3);
			p.drawLine(minSig-1,h-2,minSig+1,h-2); 
			p.drawText(minSig-(p.fontMetrics().size(SingleLine,min).width()),h-2,min);
		}
		p.drawPoint(maxSig,h-3);
		p.drawLine(maxSig-1,h-2,maxSig+1,h-2); 
		p.drawText(maxSig+2,h-2,QString::number(maxSignal));
	}
	p.setFont(font);
	if ( visibleMinMax )
		p.drawText(2,((h+font.pixelSize())>>1)-4,shortText?shortSig:sig);
	else
		p.drawText(2,((h+font.pixelSize())>>1)-1,shortText?shortSig:sig);
	p.end();
}

void QMeterBar::resizeEvent(QResizeEvent *e)
{
  QWidget::resizeEvent(e);
  stepSize = (float)(width()-2)/(maxVal-minVal);
}

/*!
    \fn QMeterBar::setSignal(int s)
*/
void QMeterBar::setSignal(int s)
{
	signal = s;
	if ( minSignal > s ) minSignal = s; 
	if ( maxSignal < s ) maxSignal = s; 
	sig = prefix+QString::number(s)+postfix;
	shortSig.setNum(s).prepend(prefix[0]+": ");
	repaint();
	QToolTip::add(this, QString(name()).append(" ")+QString::number(s)+postfix);
}

/*!
    \fn QMeterBar::setPostfix(QString p)
 */
void QMeterBar::setPostfix(QString p)
{
	postfix = p;
}

/*!
    \fn QMeterBar::setPrefix(QString p)
 */
void QMeterBar::setPrefix(QString p)
{
	prefix = p;
}


/*!
    \fn QMeterBar::calcFontSize()
 */
void QMeterBar::calcFontSize()
{
	QFontMetrics fm(font);
	prefix = name();
	prefix.append(" ");
	if (height()>10) font.setPixelSize(9);
	else font.setPixelSize(height()-2);
	if ( !(shortText = width()<70?true:false) ) {
		if ( width()-2 < fm.width(prefix+QString::number(signal)+postfix) ) {
			prefix.truncate(4);
			prefix.append("...");
		}
	}
}


/*!
    \fn QMeterBar::setFont(QString font)
 */
void QMeterBar::setFontFamily(const QString &family)
{
    font.setFamily(family);
    calcFontSize();
}


/*!
    \fn QMeterBar::setColors(QColor low, QColor middle, QColor high)
 */
void QMeterBar::setColors(const QColor &low, const QColor &middle, const QColor &high)
{
   colorLevels.lowColor = low;
   colorLevels.middleColor = middle;
   colorLevels.highColor = high;
}


/*!
    \fn QMeterBar::setColorLevels(int middle, int high)
 */
void QMeterBar::setColorLevels(int middle, int high)
{
    colorLevels.middleLevel = middle;
    colorLevels.highLevel = high;
}


/*!
    \fn QMeterBar::setFontColor(const QColor &c)
 */
void QMeterBar::setFontColor(const QColor &c)
{
    fontColor = c;
}


/*!
    \fn QMeterBar::getSignalString(bool shortText = false)
 */
QString QMeterBar::getText(bool abbr)
{
    return abbr?shortSig:sig;
}

/*!
    \fn QMeterBar::setVisibleMinMax(bool visible)
 */
void QMeterBar::setVisibleMinMax( bool visible) {
	visibleMinMax = visible;
	repaint();
}
